/*****************************************************************************/
/*                                                                           */
/*                                                                           */
/* Cube                                                                      */
/* ====                                                                      */
/*                                                                           */
/* A moving thread cube in a simple 3D room.                                 */
/*                                                                           */
/* Set graphics resolution, console mode or window mode, background colour   */  
/* and other parameters in the file "cube.cfg".                              */
/*                                                                           */
/*                                                                           */
/* Author etc                                                                */
/* ==========                                                                */
/*                                                                           */
/* Program version: cube.c 1.21                                              */
/*                                                                           */
/* The program is free to copy and modify.                                   */
/*                                                                           */
/* Author: Stefan Spaennare, Lund, Sweden                                    */
/*                                                                           */
/* E-mail: stefan@spaennare.se                                               */
/*                                                                           */
/* Home page:    http://www.spaennare.se/index.html                          */
/* Program page: http://www.spaennare.se/ssprog2.html                        */
/*                                                                           */
/* C version for DOS:     October 1994; Using Turbo C++ 2.0.                 */
/* C version for Windows: 2002-04-30;   Using Allegro game library and       */
/*                                      Microsoft Visual C++ 6.0.            */
/* Latest update:         2006-07-08                                         */
/*                                                                           */
/*                                                                           */
/* References                                                                */
/* ==========                                                                */
/*                                                                           */
/* 1. The program uses a random number generator ran2() from Numerical       */
/*    Recipes in C Second Edition.                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "allegro.h"

double pi=3.14159265358979323;

double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="cube.cfg";

char image_name[21]="cube.bmp";


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *cfgfile;

   BITMAP *bmp;
   PALETTE pal;

   int i,j,flag,q,seed,delay0;
   int xsize,ysize,col,c0;
   int bgcol,cubecol,roomcol1,roomcol2;
   int wintype,imsave;

   long idum;

   double vh,vv,sh,ch,sv,cv,w1,w2,w3,r,focus,x,y,z;
   double xs0,ys0,x1,y1,ww1,ww2,ww3,cy,cx,x2,y2,z2;
   double x0,y0,z0,dx,dy,dz,ssx1,ssx2,ssy1,ssy2,ssz1,ssz2,ss,cc;
   double xr2,yr2,zr2,speedf,xrf,yrf;
   double ttt1,ttt2,sum,totsum;
   double xtemps,ytemps,xtempr,ytempr;

   int xs[10][2];
   int ys[10][2];
   int xr[10];
   int yr[10];

   int cr[21];
   int cg[21];
   int cb[21];

   double xx[10];
   double yy[10];
   double zz[10];

   time_t tt;

   clock_t time1,timediff1;



   if ((cfgfile = fopen(cfg_name,"r")) == NULL) {
      printf("File %s not found.\n",cfg_name);
      exit(0);
   } /* if */


   fscanf(cfgfile,"%d %d",&xsize,&ysize);
   fscanf(cfgfile,"%d",&wintype);
   fscanf(cfgfile,"%d",&imsave);
   fscanf(cfgfile,"%d",&bgcol);
   fscanf(cfgfile,"%d",&cubecol);
   fscanf(cfgfile,"%d",&delay0);
   fscanf(cfgfile,"%lf",&speedf);


   if ((wintype < 0) || (wintype > 1)) {
      printf("wintype must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((bgcol < 0) || (bgcol > 1)) {
      printf("bgcol must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((imsave < 0) || (imsave > 1)) {
      printf("imsave must be 0 or 1.\n");
      exit(0);
   } /* if */


   allegro_init();

   install_keyboard();

   set_color_depth(32);


   if (wintype == 0) {
      if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */

   if (wintype == 1) {
      if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */


   set_palette(desktop_palette);


   cr[0]=0;    cg[0]=0;    cb[0]=0;
   cr[1]=255;  cg[1]=255;  cb[1]=255;

   cr[2]=255;  cg[2]=0;    cb[2]=0;
   cr[3]=0;    cg[3]=255;  cb[3]=0;
   cr[4]=0;    cg[4]=0;    cb[4]=255;

   cr[5]=255;  cg[5]=0;    cb[5]=255;
   cr[6]=255;  cg[6]=255;  cb[6]=0;
   cr[7]=0;    cg[7]=255;  cb[7]=255;

   cr[8]=128;  cg[8]=0;    cb[8]=0;
   cr[9]=0;    cg[9]=128;  cb[9]=0;
   cr[10]=0;   cg[10]=0;   cb[10]=128;

   cr[11]=128; cg[11]=128; cb[11]=0;
   cr[12]=128; cg[12]=0;   cb[12]=128;
   cr[13]=0;   cg[13]=128; cb[13]=128;

   cr[14]=128; cg[14]=128; cb[14]=128;
   cr[15]=160; cg[15]=160; cb[15]=160;
   cr[16]=192; cg[16]=192; cb[16]=192;

   cr[17]=200; cg[17]=200; cb[17]=200;
   cr[18]=55;  cg[18]=55;  cb[18]=55;


   if (bgcol == 0) {
      roomcol1=14;
      roomcol2=17;
   } /* if */

   if (bgcol == 1) {
      roomcol1=14;
      roomcol2=18;
   } /* if */


   c0=bgcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   for (i=0; i<ysize; i++) {
      line(screen,0,i,xsize-1,i,col);
   } /* for i */ 


   tt=time(0);
   ttt1=(double)(tt)+0.5;
   ttt2=fmod(ttt1,2147483647.0);
   seed=(long)(ttt2);
   idum=-seed-1;


   xrf=(double)(xsize)/640.0;
   yrf=(double)(ysize)/480.0;


   vh=0.0*pi/180.0;
   vv=0.0*pi/180.0;

   sh=sin(vh);
   ch=cos(vh);
   sv=sin(vv);
   cv=cos(vv);

   xx[1]=1;  yy[1]=1;  zz[1]=1;
   xx[2]=-1; yy[2]=1;  zz[2]=1;
   xx[3]=-1; yy[3]=-1; zz[3]=1;
   xx[4]=1;  yy[4]=-1; zz[4]=1;

   xx[5]=1;  yy[5]=1;  zz[5]=-1;
   xx[6]=-1; yy[6]=1;  zz[6]=-1;
   xx[7]=-1; yy[7]=-1; zz[7]=-1;
   xx[8]=1;  yy[8]=-1; zz[8]=-1;

   w1=0.0;
   w2=0.0;
   w3=0.0;
   ww1=speedf*pi/200.0;
   ww2=speedf*pi/200.0;
   ww3=0.0;
   focus=1000.0;
   r=40.0;
   xs0=320.0;
   ys0=240.0;
   cx=1.0;
   cy=1.0;

   x0=100.0*(ran2(&idum)-0.5);
   y0=100.0*(ran2(&idum)-0.5);
   z0=100.0*(ran2(&idum)-0.5);

   if (ran2(&idum) > 0.5) {
      dx=speedf*1.0;
   }
   else {
      dx=-speedf*1.0;
   }
   if (ran2(&idum) > 0.5) {
      dy=speedf*2.0;
   }
   else {
      dy=-speedf*2.0;
   }
   if (ran2(&idum) > 0.5) {
      dz=speedf*1.0;
   }
   else {
      dz=-speedf*1.0;
   } /* if */


   ssx1=-184.0; ssx2=184.0;
   ssy1=-539.0; ssy2=206.0;
   ssz1=-125.0; ssz2=125.0;

   xr2=227.0;
   yr2=392.0;
   zr2=170.0;

   for (i=1; i<=8; i++) {
	 x2=xx[i];
	 y2=yy[i];
	 z2=zz[i];

	 x=-xr2*x2;
	 y=yr2*y2;
	 z=zr2*z2;

	 xtempr=cx*(ch*x*(focus-y*cv)/focus+y*sh)+xs0;
	 ytempr=cy*(cv*z*(focus-y*ch)/focus+y*sv)+ys0;
	 xr[i]=(int)(xrf*xtempr+0.5);
	 yr[i]=(int)(yrf*ytempr+0.5);
   } /* for i */


   totsum=0.0;

   delay0=100*delay0;


   flag=0;
   q=1;

   do {

      for (i=1; i<=8; i++) {
	 x=xx[i];
	 y=yy[i];
	 z=zz[i];

	 ss=sin(w1); cc=cos(w1);
	 x1=x*cc+y*ss;
	 y1=y*cc-x*ss;

	 ss=sin(w3); cc=cos(w3);
	 y2=y1*cc+z*ss;
	 z2=z*cc-y1*ss;

	 ss=sin(w2); cc=cos(w2);
	 x2=x1*cc-z2*ss;
	 z2=z2*cc+x1*ss;

	 x=-r*x2-x0;
	 y=-r*y2-y0;
	 z=r*z2-z0;

	 xtemps=cx*(ch*x*(focus-y*cv)/focus+y*sh)+xs0;
	 ytemps=cy*(cv*z*(focus-y*ch)/focus+y*sv)+ys0;
	 xs[i][1]=(int)(xrf*xtemps+0.5);
	 ys[i][1]=(int)(yrf*ytemps+0.5);
      } /* for i */

      if (flag == 1) {

         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);
	 q=2;
	 line(screen,xs[1][q],ys[1][q],xs[2][q],ys[2][q],col);
	 line(screen,xs[1][q],ys[1][q],xs[5][q],ys[5][q],col);
	 line(screen,xs[1][q],ys[1][q],xs[4][q],ys[4][q],col);
	 line(screen,xs[2][q],ys[2][q],xs[3][q],ys[3][q],col);
	 line(screen,xs[2][q],ys[2][q],xs[6][q],ys[6][q],col);
	 line(screen,xs[3][q],ys[3][q],xs[7][q],ys[7][q],col);
	 line(screen,xs[3][q],ys[3][q],xs[4][q],ys[4][q],col);
	 line(screen,xs[4][q],ys[4][q],xs[8][q],ys[8][q],col);
	 line(screen,xs[5][q],ys[5][q],xs[6][q],ys[6][q],col);
	 line(screen,xs[6][q],ys[6][q],xs[7][q],ys[7][q],col);
	 line(screen,xs[7][q],ys[7][q],xs[8][q],ys[8][q],col);
	 line(screen,xs[5][q],ys[5][q],xs[8][q],ys[8][q],col);

      } /* if */

         c0=roomcol1;
         col=makecol(cr[c0],cg[c0],cb[c0]);
	 line(screen,xr[1],yr[1],xr[2],yr[2],col);
	 line(screen,xr[1],yr[1],xr[5],yr[5],col);
	 line(screen,xr[1],yr[1],xr[4],yr[4],col);
	 line(screen,xr[2],yr[2],xr[3],yr[3],col);
	 line(screen,xr[2],yr[2],xr[6],yr[6],col);
	 line(screen,xr[5],yr[5],xr[6],yr[6],col);
	 line(screen,xr[6],yr[6],xr[7],yr[7],col);
	 line(screen,xr[5],yr[5],xr[8],yr[8],col);


         c0=cubecol;
         col=makecol(cr[c0],cg[c0],cb[c0]);
	 q=1;
	 line(screen,xs[1][q],ys[1][q],xs[2][q],ys[2][q],col);
	 line(screen,xs[1][q],ys[1][q],xs[5][q],ys[5][q],col);
	 line(screen,xs[1][q],ys[1][q],xs[4][q],ys[4][q],col);
	 line(screen,xs[2][q],ys[2][q],xs[3][q],ys[3][q],col);
	 line(screen,xs[2][q],ys[2][q],xs[6][q],ys[6][q],col);
	 line(screen,xs[3][q],ys[3][q],xs[7][q],ys[7][q],col);
	 line(screen,xs[3][q],ys[3][q],xs[4][q],ys[4][q],col);
	 line(screen,xs[4][q],ys[4][q],xs[8][q],ys[8][q],col);
	 line(screen,xs[5][q],ys[5][q],xs[6][q],ys[6][q],col);
	 line(screen,xs[6][q],ys[6][q],xs[7][q],ys[7][q],col);
	 line(screen,xs[7][q],ys[7][q],xs[8][q],ys[8][q],col);
	 line(screen,xs[5][q],ys[5][q],xs[8][q],ys[8][q],col);

         c0=roomcol2;
         col=makecol(cr[c0],cg[c0],cb[c0]);
	 line(screen,xr[3],yr[3],xr[7],yr[7],col);
	 line(screen,xr[3],yr[3],xr[4],yr[4],col);
	 line(screen,xr[4],yr[4],xr[8],yr[8],col);
	 line(screen,xr[7],yr[7],xr[8],yr[8],col);


      for (i=1; i<=8; i++) {
	 xs[i][2]=xs[i][1];
	 ys[i][2]=ys[i][1];
      } /* for i */

      flag=1;

      w1=w1+ww1;
      w2=w2+ww2;
      w3=w3+ww3;

      x0=x0+dx;
      y0=y0+dy;
      z0=z0+dz;


      if ((x0 < ssx1) || (x0 > ssx2)) {
	 dx=-dx;
	 ww1=-ww1;
      }

      if ((y0 < ssy1) || (y0 > ssy2)) {
	 dy=-dy;
	 ww3=-ww3;
      }

      if ((z0 < ssz1) || (z0 > ssz2)) {
	 dz=-dz;
	 ww2=-ww2;
      }

      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      totsum=totsum+sum;

   } while (!keypressed());



   if (imsave == 1) {
      get_palette(pal);
      bmp=create_sub_bitmap(screen,0,0,xsize,ysize);
      save_bitmap(image_name,bmp,pal);
      destroy_bitmap(bmp);
   } /* if */

   allegro_exit();

   fclose(cfgfile);

   printf("dummy = %12.6f\n",totsum);

   return(0);

} /* End  */

END_OF_MAIN();
